@@ -52,6 +52,7 @@ gdk_colormap_finalize (GObject *object)
   
   g_free (private->info);
   g_free (colormap->colors);
+  g_free (private);
 
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
@@ -202,7 +203,7 @@ gdk_colormap_new (GdkVisual *visual,
 }
 
 GdkColormap*
-gdk_colormap_get_system (void)
+gdk_screen_get_system_colormap (GdkScreen *screen)
 {
   static GdkColormap *colormap = NULL;
 
@@ -854,3 +855,11 @@ gdk_colormap_query_color (GdkColormap *colormap,
       break;
     }
 }
+
+GdkScreen*
+gdk_colormap_get_screen (GdkColormap *cmap)
+{
+  g_return_val_if_fail (cmap != NULL, NULL);
+
+  return gdk_screen_get_default ();
+}
