@@ -659,6 +659,26 @@ get_real_window (XEvent *event)
     }
 }
 
+#ifdef G_ENABLE_DEBUG
+static const char notify_modes[][18] = {
+  "NotifyNormal",
+  "NotifyGrab",
+  "NotifyUngrab",
+  "NotifyWhileGrabbed"
+};
+
+static const char notify_details[][22] = {
+  "NotifyAncestor",
+  "NotifyVirtual",
+  "NotifyInferior",
+  "NotifyNonlinear",
+  "NotifyNonlinearVirtual",
+  "NotifyPointer",
+  "NotifyPointerRoot",
+  "NotifyDetailNone"
+};
+#endif
+
 static gboolean
 gdk_event_translate (GdkDisplay *display,
 		     GdkEvent   *event,
@@ -668,6 +688,7 @@ gdk_event_translate (GdkDisplay *display,
   
   GdkWindow *window;
   GdkWindowObject *window_private;
+  GdkWindow *filter_window;
   GdkWindowImplX11 *window_impl = NULL;
   gint return_val;
   gint xoffset, yoffset;
@@ -722,6 +743,14 @@ gdk_event_translate (GdkDisplay *display,
   window = gdk_window_lookup_for_display (display, xwindow);
   window_private = (GdkWindowObject *) window;
 
+  /* We always run the filters for the window where the event
+   * is delivered, not the window that it relates to
+   */
+  if (xevent->xany.window == xwindow)
+    filter_window = window;
+  else
+    filter_window = gdk_window_lookup_for_display (display, xevent->xany.window);
+
   if (window)
     {
       screen = GDK_WINDOW_SCREEN (window);
@@ -771,17 +800,26 @@ gdk_event_translate (GdkDisplay *display,
 	  goto done;
 	}
     }
-  else if (window_private)
+  else if (filter_window)
     {
       /* Apply per-window filters */
+      GdkWindowObject *filter_private = (GdkWindowObject *) filter_window;
       GdkFilterReturn result;
-      result = gdk_event_apply_filters (xevent, event,
-					window_private->filters);
-      
-      if (result != GDK_FILTER_CONTINUE)
+
+      if (filter_private->filters)
 	{
-	  return_val = (result == GDK_FILTER_TRANSLATE) ? TRUE : FALSE;
-	  goto done;
+	  g_object_ref (filter_window);
+	  
+	  result = gdk_event_apply_filters (xevent, event,
+					    filter_private->filters);
+	  
+	  g_object_unref (filter_window);
+      
+	  if (result != GDK_FILTER_CONTINUE)
+	    {
+	      return_val = (result == GDK_FILTER_TRANSLATE) ? TRUE : FALSE;
+	      goto done;
+	    }
 	}
     }
       
@@ -1034,10 +1072,10 @@ gdk_event_translate (GdkDisplay *display,
       if (window &&
 	  GDK_WINDOW_TYPE (window) != GDK_WINDOW_CHILD &&
 	  xevent->xcrossing.detail != NotifyInferior &&
-	  xevent->xcrossing.focus && !window_impl->has_focus)
+	  xevent->xcrossing.focus && !window_impl->has_focus_window)
 	{
 	  gboolean had_focus = HAS_FOCUS (window_impl);
-	  
+
 	  window_impl->has_pointer_focus = TRUE;
 
 	  if (HAS_FOCUS (window_impl) != had_focus)
@@ -1129,12 +1167,12 @@ gdk_event_translate (GdkDisplay *display,
       if (window &&
 	  GDK_WINDOW_TYPE (window) != GDK_WINDOW_CHILD &&
 	  xevent->xcrossing.detail != NotifyInferior &&
-	  xevent->xcrossing.focus && !window_impl->has_focus)
+	  xevent->xcrossing.focus && !window_impl->has_focus_window)
 	{
 	  gboolean had_focus = HAS_FOCUS (window_impl);
 	  
 	  window_impl->has_pointer_focus = FALSE;
-
+	  
 	  if (HAS_FOCUS (window_impl) != had_focus)
 	    generate_focus_event (window, FALSE);
 	}
@@ -1207,7 +1245,10 @@ gdk_event_translate (GdkDisplay *display,
        */
     case FocusIn:
       GDK_NOTE (EVENTS,
-		g_message ("focus in:\t\twindow: %ld", xevent->xfocus.window));
+		g_message ("focus in:\t\twindow: %ld, detail: %s, mode: %s",
+			   xevent->xfocus.window,
+			   notify_details[xevent->xfocus.detail],
+			   notify_modes[xevent->xfocus.mode]));
       
       if (window && GDK_WINDOW_TYPE (window) != GDK_WINDOW_CHILD)
 	{
@@ -1219,6 +1260,7 @@ gdk_event_translate (GdkDisplay *display,
 	    case NotifyNonlinear:
 	    case NotifyVirtual:
 	    case NotifyNonlinearVirtual:
+	      window_impl->has_focus_window = TRUE;
 	      /* We pretend that the focus moves to the grab
 	       * window, so we pay attention to NotifyGrab
 	       * NotifyUngrab, and ignore NotifyWhileGrabbed
@@ -1246,8 +1288,11 @@ gdk_event_translate (GdkDisplay *display,
       break;
     case FocusOut:
       GDK_NOTE (EVENTS,
-		g_message ("focus out:\t\twindow: %ld", xevent->xfocus.window));
-
+		g_message ("focus out:\t\twindow: %ld, detail: %s, mode: %s",
+			   xevent->xfocus.window,
+			   notify_details[xevent->xfocus.detail],
+			   notify_modes[xevent->xfocus.mode]));
+      
       if (window && GDK_WINDOW_TYPE (window) != GDK_WINDOW_CHILD)
 	{
 	  gboolean had_focus = HAS_FOCUS (window_impl);
@@ -1258,6 +1303,7 @@ gdk_event_translate (GdkDisplay *display,
 	    case NotifyNonlinear:
 	    case NotifyVirtual:
 	    case NotifyNonlinearVirtual:
+	      window_impl->has_focus_window = FALSE;
 	      if (xevent->xfocus.mode != NotifyWhileGrabbed)
 		window_impl->has_focus = FALSE;
 	      break;
