@@ -1072,19 +1072,24 @@ gtk_item_factory_create_item (GtkItemFactory	     *ifactory,
 
   if (GTK_IS_RADIO_MENU_ITEM (widget))
     gtk_radio_menu_item_set_group (GTK_RADIO_MENU_ITEM (widget), radio_group);
-  if (GTK_IS_CHECK_MENU_ITEM (widget))
-    gtk_check_menu_item_set_show_toggle (GTK_CHECK_MENU_ITEM (widget), TRUE);
   if (type_id == quark_type_image_item)
     {
+      GError *error = NULL;
       GdkPixbuf *pixbuf = NULL;
 
       image = NULL;
       pixbuf = gdk_pixbuf_new_from_inline (-1,
 					   entry->extra_data,
 					   FALSE,
-					   NULL);
+					   &error);
       if (pixbuf)
 	image = gtk_image_new_from_pixbuf (pixbuf);
+      else
+	{
+	  g_warning ("Error loading menu image: %s", error->message);
+	  g_error_free (error);
+	}
+	    
       if (image)
 	{
 	  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (widget), image);
@@ -1465,7 +1470,7 @@ ifactory_delete_popup_data (GtkObject	   *object,
  * @x: the x position 
  * @y: the y position
  * @mouse_button: the mouse button which was pressed to initiate the popup
- * @time: the time at which the activation event occurred
+ * @time_: the time at which the activation event occurred
  *
  * Pops up the menu constructed from the item factory at (@x, @y).
  *
@@ -1474,11 +1479,11 @@ ifactory_delete_popup_data (GtkObject	   *object,
  * a mouse button press, such as a mouse button release or a keypress,
  * @mouse_button should be 0.
  *
- * The @time parameter should be the time stamp of the event that
+ * The @time_ parameter should be the time stamp of the event that
  * initiated the popup. If such an event is not available, use
  * gtk_get_current_event_time() instead.
  *
- * The operation of the @mouse_button and the @time parameter is the same
+ * The operation of the @mouse_button and the @time_ parameter is the same
  * as the @button and @activation_time parameters for gtk_menu_popup().
  */
 void
@@ -1500,7 +1505,7 @@ gtk_item_factory_popup (GtkItemFactory		*ifactory,
  * @x: the x position 
  * @y: the y position
  * @mouse_button: the mouse button which was pressed to initiate the popup
- * @time: the time at which the activation event occurred
+ * @time_: the time at which the activation event occurred
  *
  * Pops up the menu constructed from the item factory at (@x, @y). Callbacks
  * can access the @popup_data while the menu is posted via 
@@ -1511,11 +1516,11 @@ gtk_item_factory_popup (GtkItemFactory		*ifactory,
  * a mouse button press, such as a mouse button release or a keypress,
  * @mouse_button should be 0.
  *
- * The @time parameter should be the time stamp of the event that
+ * The @time_ parameter should be the time stamp of the event that
  * initiated the popup. If such an event is not available, use
  * gtk_get_current_event_time() instead.
  *
- * The operation of the @mouse_button and the @time parameters is the same
+ * The operation of the @mouse_button and the @time_ parameters is the same
  * as the @button and @activation_time parameters for gtk_menu_popup().
  */
 void
