@@ -124,8 +124,8 @@ gtk_text_render_state_new (GtkWidget    *widget,
 static void
 gtk_text_render_state_destroy (GtkTextRenderState *state)
 {
-  gdk_gc_unref (state->fg_gc);
-  gdk_gc_unref (state->bg_gc);
+  g_object_unref (state->fg_gc);
+  g_object_unref (state->bg_gc);
 
   g_free (state);
 }
@@ -395,22 +395,22 @@ render_layout_line (GdkDrawable        *drawable,
                                               pixbuf_rect.x, pixbuf_rect.y);
                     }
 
-                  gdk_pixbuf_render_to_drawable (pixbuf,
-                                                 drawable,
-                                                 render_state->fg_gc,
-                                                 draw_rect.x - pixbuf_rect.x,
-                                                 draw_rect.y - pixbuf_rect.y,
-                                                 draw_rect.x, draw_rect.y,
-                                                 draw_rect.width,
-                                                 draw_rect.height,
-                                                 GDK_RGB_DITHER_NORMAL,
-                                                 0, 0);
+                  gdk_draw_pixbuf (drawable,
+				   render_state->fg_gc,
+				   pixbuf,
+				   draw_rect.x - pixbuf_rect.x,
+				   draw_rect.y - pixbuf_rect.y,
+				   draw_rect.x, draw_rect.y,
+				   draw_rect.width,
+				   draw_rect.height,
+				   GDK_RGB_DITHER_NORMAL,
+				   0, 0);
 
                   if (mask)
                     {
                       gdk_gc_set_clip_rectangle (render_state->fg_gc,
                                                  &render_state->clip_rect);
-                      g_object_unref (G_OBJECT (mask));
+                      g_object_unref (mask);
                     }
                 }
 
@@ -426,7 +426,7 @@ render_layout_line (GdkDrawable        *drawable,
 
               if (widgets)
                 {
-                  g_object_ref (G_OBJECT (widget));
+                  g_object_ref (widget);
                   *widgets = g_list_prepend (*widgets, widget);
                 }
             }
