@@ -422,6 +422,8 @@ gtk_tree_selection_get_selected (GtkTreeSelection  *selection,
  * </programlisting></informalexample>
  *
  * Return value: A #GList containing a #GtkTreePath for each selected row.
+ *
+ * Since: 2.2
  **/
 GList *
 gtk_tree_selection_get_selected_rows (GtkTreeSelection   *selection,
@@ -543,6 +545,8 @@ gtk_tree_selection_count_selected_rows_helper (GtkRBTree *tree,
  * Returns the number of rows that have been selected in @tree.
  *
  * Return value: The number of rows selected.
+ * 
+ * Since: 2.2
  **/
 gint
 gtk_tree_selection_count_selected_rows (GtkTreeSelection *selection)
@@ -610,7 +614,7 @@ gtk_tree_selection_selected_foreach (GtkTreeSelection            *selection,
   GtkTreeIter iter;
 
   guint inserted_id, deleted_id, reordered_id;
-  gboolean stop = FALSE, has_next = FALSE, has_parent = FALSE;
+  gboolean stop = FALSE, has_next = TRUE, has_parent = TRUE;
 
   g_return_if_fail (GTK_IS_TREE_SELECTION (selection));
   g_return_if_fail (selection->tree_view != NULL);
@@ -681,7 +685,7 @@ gtk_tree_selection_selected_foreach (GtkTreeSelection            *selection,
 	  gtk_tree_path_append_index (path, 0);
 
 	  /* we do the sanity check at the bottom of this function */
-	  if (has_child)
+	  if (!has_child)
 	    goto out;
 	}
       else
@@ -698,8 +702,9 @@ gtk_tree_selection_selected_foreach (GtkTreeSelection            *selection,
 		  done = TRUE;
 		  gtk_tree_path_next (path);
 
-		  /* Sanity Check! */
-		  TREE_VIEW_INTERNAL_ASSERT_VOID (has_next);
+		  /* we do the sanity check at the bottom of this function */
+		  if (!has_next)
+		    goto out;
 		}
 	      else
 		{
@@ -720,7 +725,7 @@ gtk_tree_selection_selected_foreach (GtkTreeSelection            *selection,
 		  gtk_tree_path_up (path);
 
 		  /* we do the sanity check at the bottom of this function */
-		  if (has_parent)
+		  if (!has_parent)
 		    goto out;
 		}
 	    }
@@ -741,9 +746,9 @@ out:
                                reordered_id);
 
   /* check if we have to spew a scary message */
-  if (has_next)
+  if (!has_next)
     TREE_VIEW_INTERNAL_ASSERT_VOID (has_next);
-  if (has_parent)
+  if (!has_parent)
     TREE_VIEW_INTERNAL_ASSERT_VOID (has_parent);
   if (stop)
     g_warning
@@ -1146,6 +1151,7 @@ gtk_tree_selection_real_modify_range (GtkTreeSelection *selection,
 {
   GtkRBNode *start_node, *end_node;
   GtkRBTree *start_tree, *end_tree;
+  GtkTreePath *anchor_path = NULL;
   gboolean dirty = FALSE;
 
   switch (gtk_tree_path_compare (start_path, end_path))
@@ -1159,6 +1165,7 @@ gtk_tree_selection_real_modify_range (GtkTreeSelection *selection,
 				start_path,
 				&end_tree,
 				&end_node);
+      anchor_path = end_path;
       break;
     case 0:
       _gtk_tree_view_find_node (selection->tree_view,
@@ -1167,6 +1174,7 @@ gtk_tree_selection_real_modify_range (GtkTreeSelection *selection,
 				&start_node);
       end_tree = start_tree;
       end_node = start_node;
+      anchor_path = start_path;
       break;
     case -1:
       _gtk_tree_view_find_node (selection->tree_view,
@@ -1177,12 +1185,24 @@ gtk_tree_selection_real_modify_range (GtkTreeSelection *selection,
 				end_path,
 				&end_tree,
 				&end_node);
+      anchor_path = start_path;
       break;
     }
 
   g_return_val_if_fail (start_node != NULL, FALSE);
   g_return_val_if_fail (end_node != NULL, FALSE);
 
+  if (anchor_path)
+    {
+      if (selection->tree_view->priv->anchor)
+	gtk_tree_row_reference_free (selection->tree_view->priv->anchor);
+
+      selection->tree_view->priv->anchor =
+	gtk_tree_row_reference_new_proxy (G_OBJECT (selection->tree_view),
+	                                  selection->tree_view->priv->model,
+					  anchor_path);
+    }
+
   do
     {
       dirty |= gtk_tree_selection_real_select_node (selection, start_tree, start_node, (mode == RANGE_SELECT)?TRUE:FALSE);
@@ -1241,6 +1261,8 @@ gtk_tree_selection_select_range (GtkTreeSelection *selection,
  *
  * Unselects a range of nodes, determined by @start_path and @end_path
  * inclusive.
+ *
+ * Since: 2.2
  **/
 void
 gtk_tree_selection_unselect_range (GtkTreeSelection *selection,
